@@ -8,12 +8,32 @@ import coinstac_dinunet.utils.tensorutils as _tu
 import torch as _torch
 
 
-def _load(state, site, site_vars):
-    grads_file = state['baseDirectory'] + _os.sep + site + _os.sep + site_vars['grads_file']
+def _multi_load(file_key, state, site, site_vars):
+    grads_file = state['baseDirectory'] + _os.sep + site + _os.sep + site_vars[file_key]
     return _tu.load_arrays(grads_file)
 
 
 class COINNReducer:
+
+    def _load(self, file_key):
+        return list(
+            self.pool.starmap(
+                _partial(_multi_load, file_key, self.state), self.input.items()
+            )
+        )
+
+    def _average(self, file_key):
+        sites_data = self._load(file_key)
+        gpu_data = []
+        for data in list(zip(*sites_data)):
+            gpu_data.append(_torch.from_numpy(_np.array(data)).to(self.device, non_blocking=True).mean(0))
+
+        avg_data = []
+        for data in gpu_data[::-1]:
+            avg_data.append(data.cpu().numpy().astype(self.dtype))
+
+        return avg_data[::-1]
+
     def __init__(self, trainer, mp_pool, **kw):
         self.cache = trainer.cache
         self.input = trainer.input
@@ -25,22 +45,12 @@ class COINNReducer:
 
     def reduce(self):
         """ Average each sites gradients and pass it to all sites. """
-        out = {'avg_grads_file': _conf.avg_grads_file}
-
-        grads = list(
-            self.pool.starmap(
-                _partial(_load, self.state), self.input.items()
-            )
-        )
-
-        avg_grads = []
-        for data in list(zip(*grads)):
-            data = _torch.from_numpy(_np.array(data)).to(self.device).mean(0)
-            avg_grads.append(data.cpu().numpy().astype(self.dtype))
-
+        out = {
+            'avg_grads_file': _conf.avg_grads_file
+        }
         _tu.save_arrays(
             self.state['transferDirectory'] + _os.sep + out['avg_grads_file'],
-            _np.array(avg_grads, dtype=object)
+            _np.array(self._average('grads_file'), dtype=object)
         )
 
         out['update'] = True
